/*
 * XORText.java
 *
 * Created on June 18, 2002, 5:06 PM
 */

/**
 *
 * @author  Dmitri Pavlenkov
 * @version 1.0
 * Copyright 2002 (c) Dmitri Pavlenkov
 * Use allowed under Common Public Lincense
 * Support to author is appreciated
 * Send your gratitude to:
 * Dmitri Pavlenkov
 * 5300 Barrow Dr
 * St. James City, FL, 33956
 * USA
 */
import java.io.*;
import java.util.*;
import java.text.*;
import net.fractalkiss.util.*;

/** Cube alignment application and tools for working with text. Works a little differently from FramePlane, here -frame0 option causes looped processing of entire file.
 *
 */
public class FrameCube extends Util {
  
  static DecimalFormat d3=new DecimalFormat("000");
  /** Creates new FrameCube
   */
  public FrameCube() {
  }
  
  /** parses text and aligns cubes
   * @param args the command line arguments
   */
  static void printHelp() {
    System.out.println("\nUsage: java FrameCube [-option] res textfile [alignfile [alignout]]");
    System.out.println("Options (only one allowed):");
    System.out.println("-h print this message");
    System.out.println("-frameXXX specify frame size, if omitted or XXX is 0 perform");
    System.out.println("          single frame alignment");
    System.out.println("Explain:");
    System.out.println("res       - normally 128 for text or 256 for all other input");
    System.out.println("textfile  - either plain or framed input");
    System.out.println("alignfile - source alignment cube (defaults to \"align.acube\")");
    System.out.println("alignout  - write alignment cube (defaults to \"align.acube\")\n");
  }
  /** java FrameCube -h
   * @param args java FrameCube -h
   */  
  public static void main(String args[]) {
    InputStream in=null;
    if(args.length<1||args[0].indexOf("-h")>-1) {
      printHelp();
      System.exit(0);
    }
    int position=1;
    int frame=0;
    boolean align=true;
    if(args[0].indexOf("-frame")>-1) {
      position=2;
      String skip="-frame";
      try {
        frame=Integer.parseInt(args[0].substring(args[0].indexOf(skip)+skip.length()));
      } catch (Exception e) {
        printHelp();
        System.out.println("-frameXXX parse error, try to put a number after \"-frame\"");
        System.exit(0);
      }
    }
    String workfile="work.acube";
    String alignfile="align.acube";
    int res=128; //resolution
    try {
      res=Integer.parseInt(args[position-1]);
    } catch (Exception e) {
      printHelp();
      System.out.println("res parse error, try to put a number there");
      System.exit(0);
    }
    if(args.length>position) {
      try {
        in=new FileInputStream(args[position]);
        position++;
      } catch (Exception e) {
        //e.printStackTrace();
        printHelp();
        System.out.println(e.getMessage()+"\ntextfile \""+args[position]+"\" could not be opened");
        System.exit(1);
      }
    } else {
      printHelp();
      System.out.println("textfile parameter is required");
      System.exit(0);
    }
    if(args.length>position) {
      alignfile=args[position];
      position++;
    }
    String alignout=alignfile;
    if(args.length>position) {
      alignout=args[position];
    }
    char[][][] cube=new char [res][res][res];
    if(!readCube(cube, alignfile, res)) fill03(cube, res, res, res);
    if(frame==0) {
      int[] param=parse(in, workfile,res);
      //System.exit(0);
      System.out.println("Text lengh: "+param[0]);
      System.out.println("Size of workfile (length*stretch): "+param[1]*param[0]);
      align(cube, workfile, param[0], param[1]);
      writeCube(cube, alignout, res);
      System.exit(0);
    }
    try {
      alignFrames(cube,res, frame, in, alignout);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  private static int levels=8;
  private static Fragger fr=new Fragger(levels);
  /** Performs frame fragmentation
   * @param cube 3D char array
   * @param res resolution 128 or 256
   * @param frame bytes per frame
   * @param in stream of framed input
   * @param name stored cube if present
   * @throws Exception file I/O exceptions
   */  
  public static void alignFrames(char[][][] cube, int res, int frame, InputStream in, String name) throws Exception {
    int dot=0;
    int c=-1;
    char[] current=new char[frame];
    while((c=in.read())>-1) {
      if(c>res-1) continue;
      current[dot]=(char)c;
      dot++;
      if(dot%frame==0) {
        dot=0;
        if(fr.nextFrame(current)) fr.workCube(cube,frame);
        current=new char[frame];
      }
    }
    writeCube(cube,name,res);
  }
  /** Reserved for looping inputstream, not implemented
   * @param looped char array, records frame*levels chars from beginning
   * @param c character to write
   * @param counter position to write
   */  
  public static void loopRead(char[] looped, int c, int counter) {
  
  }
  /** Reads a char <CODE>cube</CODE> of size <CODE>res*res*res</CODE> from the <CODE>name</CODE> file
   *
   * @param cube 3D array to store the cube
   * @param name file to read
   * @param res size of cube sides
   * @return returns false if file can't be opened
   */
  public static boolean readCube(char[][][] cube, String name, int res) {
    try {
      FileInputStream align=new FileInputStream(name);
      for(int x=0;x<res;x++) {
        for(int y=0;y<res;y++) {
          for(int z=0;z<res;z++) {
            char b=(char)align.read();
            cube[x][y][z]=b;
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }
  /** writes <CODE>cube</CODE> to <CODE>name</CODE> file
   * @param cube 3D char array to write
   * @param name file name
   * @param res size of cube sides
   */
  public static void writeCube(char[][][] cube, String name, int res) {
    try {
      FileOutputStream align=new FileOutputStream(name);
      for(int x=0;x<res;x++) {
        for(int y=0;y<res;y++) {
          for(int z=0;z<res;z++) {
            align.write(cube[x][y][z]);
          }
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  /** performs XOR operation on every char with the next in the array, the last char XORs with the first
   * @param sb char array
   * @return char array
   */
  public static char[] xorNext(char[] sb) {
    int len=sb.length;
    char[] xor=new char[len];
    for(int i=1;i<=len;i++) {
      xor[i-1]=(char)(sb[i%len]^sb[i-1]);
    }
    return xor;
  }
  /** checks if to char arrays have the same value
   * @param bef char array before NANDing
   * @param aft char array after NANDing
   * @return returns true if same, false otherwise
   */
  public static boolean repeat(char[] bef, char[] aft) {
    for(int i=0;i<bef.length;i++) {
      if((((int)bef[i])-((int)aft[i]))!=0) {
        //System.out.println("column: "+((i+1)*9));
        return false;
      }
    }
    return true;
  }
  /** Displays spaced 3-digit decimal presentation of char array
   * @param bin char array
   * @return String of spaced 3-digit decimal numbers
   */
  public static String showDecimal(char [] bin) {
    StringBuffer bf=new StringBuffer();
    for(int i=0;i<bin.length;i++) {
      bf.append(d3.format((int)bin[i])+" ");
    }
    return new String(bf);
  }
  /** Displays spaced 8-bit presentation of char array
   * @param bin char array
   * @return String of '*'s and '-'s for 1 and 0 respectively
   */
  public static String showBinary(char [] bin) {
    StringBuffer bf=new StringBuffer();
    for(int i=0;i<bin.length;i++) {
      char mask=(char)(1<<7);
      for(int m=0;m<8;m++) {
        bf.append((mask&bin[i])==0?'-':'*');
        mask>>=1;
      }
      bf.append(' ');
    }
    return new String(bf);
  }
  /** Parses text and writes 9 layers of increasing fragmenticity.
   * <PRE>1st layer is text,
   * 2nd layer frag[0] = xor[0] NAND text
   *   xor[0]  = text XOR loopLeft(text)
   * 3rd layer frag[1] = xor[1] NAND xor[0] NAND text
   *   xor[1]  = xor[0] XOR loopLeft xor[0]
   * ...
   * 9th layer frag[7] = xor[7] NAND ... xor[0] NAND text
   * X NAND Y = NOT X AND Y : key fractal operation
   * XOR operation on FRAMEs calculates overlaps
   * NAND operation on overlaps and FRAMEs calculates fragments
   * NAND operation on overlap(n+1) and overlap(n) calculates fragment shapes
   * IXOR is the iterative XOR on frames, essentially it cleans overlaps
   * IXOR(n,b) = IXOR(n-1,b) XOR IXOR(n-1,b plus 1)
   * IXOR(0,b) = FRAME(b) XOR FRAME(b plus 1)
   * FRAG calculates n increasingly fragmented copies of a given frame b
   * FRAG(n,b) = INAND (n-1,b) NAND FRAME(b)
   * INAND is iterative NAND of one xor mask with the previous from left to right
   * INAND(n,b) = IXOR(n,b) NAND IXOR(n-1,b) NAND ... IXOR(0,b)
   * </PRE>
   * if we view text as one frame, then looping text left by one letter will
   * produce more frames. By looping xor mask we'll model consecutive xors of
   * one frame with the next. <CODE>parse</CODE> frags only top frame (text itself)
   * producing 8 copies <CODE>FRAG(0,0-7)</CODE> because text is repetitive.
   * Those copies are, of course, of same length and are layered one after another
   * below the text itself.
   *
   * @return returns int[2] parameters
   * param[0] is length of text
   * param[1] is number of copies with text, normally 9
   * @param res resolution 128 or 256
   * @param in reader text stream
   * @param name workfile to write, "work.acube" is default
   */
  public static int[] parse(InputStream in, String name, int res) {
    StringBuffer bf=new StringBuffer();
    int b=0;
    // Read text file
    try {
      // Skip overfluous chars and downgrade to space=0
      while ((b=in.read())>-1) {
        //b=Character.toUpperCase((char)b);
        if(b<res)
          bf.append((char)b);
        //else
        //bf.append(' ');
        
        
      }
    }
    // Handle read errors
    catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    } finally {
      // Close stream on error or final
      try {
        in.close();
        // Handle close error
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    char[] sb=new char[bf.length()];
    for(int i=0;i<bf.length();i++) {
      //System.out.print("    "+bf.charAt(i)+"    ");
    }
    System.out.println();
    bf.getChars(0,bf.length(),sb,0);
    bf=null;
    //sb=minus32(sb);
    OutputStream ou=null;
    try {
      ou=new FileOutputStream("work.acube");
    } catch (Exception e) {e.printStackTrace();}
    //System.out.println(showBinary(sb));
    //System.out.println(showBinary(nand(sb,xorNext(sb))));
    //System.out.println(showBinary(xorNext(sb)));
    writeBytes(ou,sb);
    writeBytes(ou,nand(sb,xorNext(sb)));
    ArrayList frag=new ArrayList();
    frag.add(0,xorNext(sb));
    char[] fr=null;
    char[] bef=null;
    int brk=0;
    L:  for(int i=1;i<levels;i++) {
      frag.add(i,xorNext((char[])frag.get(i-1)));
      fr=nand((char[])frag.get(i-1),(char[])frag.get(i));
      for(int j=i-1;j>0;j--) {
        fr=nand((char[])frag.get(j-1),fr);
      }
      char[] aft=nand(sb,fr);
      brk=i+1;
      if(bef!=null) {
        if(repeat(bef,aft)) {
          break L;
        }
      }
      //System.out.println(showBinary((char[])frag.get(i)));
      //System.out.println(showBinary(aft));
      writeBytes(ou,aft);
      bef=aft;
    }
    frag.clear();frag=null;fr=null;bef=null;
    try {
      ou.flush();
      ou.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
    int len=sb.length; sb=null;
    int[] param=new int[2];
    param[0]=len;
    param[1]=brk;
    return param;
  }
  /** Performs so-called "alignment" by processing fragmented sets.
   * Depending on how many bits are in your input and how many
   * different values your input allows, your cube may have different
   * dimensions. For instance, if you process english text, then
   * you only need first half of ASCII table, which is 128 characters,
   * All your characters are numbered from 0 to 127, which allows
   * you to use character codes as indexes. And since every byte can
   * set 8 bits, you only need 8 fragmented copies every time.
   * This function works with 128 bytes per side. let's say you have
   * cube[x][y][z], index x is your character code, index y is the byte
   * that happens to be in any of the 8 copies under your character,
   * index z is the next n-th character code, where n is the copy number
   * or fragmentation level, which is one of the 8. cube[x][y][z] value
   * has every bit set according to on which level y occured
   * under character x and pointed to character z, if you nuber your
   * levels n=0..7 then the bit is set by
   * <PRE><CODE>cube[x][y][z] = cube[x][y][z] | 1 shiftLeft n</CODE>
   * </PRE>Now, it is enough to align text once, because of its inherent periodicity,
   * but you need to perform alignment on every frame of other kinds of input, like
   * video and audio. For every frame calculate 8 fragments and get to work. The alignment
   * cube will grow with periodicity records.
   *
   * @param cube 3D char array
   * @param name stored cube file
   * @param len file length
   * @param brk number of fragments
   */  
  public static void align(char[][][] cube, String name, int len, int brk) {
    RandomAccessFile work=null;
    try {
      work=new RandomAccessFile(name,"r");
    } catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
    // index 0 records chars by position, 1 position by chars, 2 original
    // position by chars
    try {
      for(int pos=0;pos<len;pos++) {
        for(int d=2;d<brk;d++) {
          work.seek(len*d+pos);
          char c=(char)work.read();
          work.seek(len*(d-1)+(pos+1)%len);
          char level=(char)work.read();
          work.seek(len*(d-2)+(pos+2)%len);
          char levelT=(char)work.read();
          cube[c][level][levelT]|=(1<<(d-2));
          if(d<4) {
            cube[levelT][level][c]|=(1<<(brk-4+d));
          }
        }
      }
      work.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
