/*
 * XORText.java
 *
 * Created on June 18, 2002, 5:06 PM
 */

/**
 *
 * @author  Dmitri Pavlenkov
 * @version 1.0
 * Copyright 2002 (c) Dmitri Pavlenkov
 * Use allowed under Common Public Lincense
 * Support to author is appreciated
 * Send your gratitude to:
 * Dmitri Pavlenkov
 * 5300 Barrow Dr
 * St. James City, FL, 33956
 * USA
 */
import java.io.*;
import java.util.*;
import java.text.*;
import net.fractalkiss.util.*;

/** Plane alignment application and tools for working with text and other input.
 */
public class FramePlane extends Util {
  
  static DecimalFormat d3=new DecimalFormat("000");
  /** Creates new FramePlane
   */
  public FramePlane() {
  }
  
  /** parses text and aligns cubes
   * @param args the command line arguments
   */
  static void printHelp() {
    System.out.println("\nUsage: java FramePlane [-option] res textfile [alignfile [alignout]]");
    System.out.println("Options (only one allowed):");
    System.out.println("-h print this message");
    System.out.println("-frameXXX specify frame size, if omitted or XXX is 1 perform");
    System.out.println("          single frame alignment");
    System.out.println("Explain:");
    System.out.println("res       - normally 128 for text or 256 for all other input");
    System.out.println("textfile  - either plain or framed input");
    System.out.println("alignfile - source alignment cube (defaults to \"align.aplane\")");
    System.out.println("alignout  - write alignment cube (defaults to \"align.aplane\")\n");
  }
  /** java FramePlane -h
   * @param args java FrameCube -h
   */
  public static void main(String args[]) {
    InputStream in=null;
    if(args.length<1||args[0].indexOf("-h")>-1) {
      printHelp();
      System.exit(0);
    }
    int position=1;
    int frame=1;
    boolean align=true;
    if(args[0].indexOf("-frame")>-1) {
      position=2;
      String skip="-frame";
      try {
        frame=Integer.parseInt(args[0].substring(args[0].indexOf(skip)+skip.length()));
      } catch (Exception e) {
        printHelp();
        System.out.println("-frameXXX parse error, try to put a number after \"-frame\"");
        System.exit(0);
      }
    }
    if(frame<1) frame=1;
    String workfile="work.acube";
    String alignfile="align.aplane";
    int res=128; //resolution
    try {
      res=Integer.parseInt(args[position-1]);
    } catch (Exception e) {
      printHelp();
      System.out.println("res parse error, try to put a number there");
      System.exit(0);
    }
    if(args.length>position) {
      try {
        in=new FileInputStream(args[position]);
        position++;
      } catch (Exception e) {
        //e.printStackTrace();
        printHelp();
        System.out.println(e.getMessage()+"\ntextfile \""+args[position]+"\" could not be opened");
        System.exit(1);
      }
    } else {
      printHelp();
      System.out.println("textfile parameter is required");
      System.exit(0);
    }
    if(args.length>position) {
      alignfile=args[position];
      position++;
    }
    String alignout=alignfile;
    if(args.length>position) {
      alignout=args[position];
    }
    char[][] plane=new char [res][res];
    if(!readPlane(plane, alignfile, res)) fill02(plane, res, res);
    try {
      alignFrames(plane,res, frame, in, alignout);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  private static int levels=8;
  private static Fragger fr=new Fragger(levels);
  /** Performs frame fragmentation
   * @param plane 2D char array
   * @param res resolution 128 or 256
   * @param frame bytes per frame
   * @param in stream of framed input
   * @param name stored cube if present
   * @throws Exception file I/O exceptions
   */
  public static void alignFrames(char[][] plane, int res, int frame, InputStream in, String name) throws Exception {
    int dot=0;
    int c=-1;
    char[] current=new char[frame];
    while((c=in.read())>-1) {
      if(c>res-1) continue;
      current[dot]=(char)c;
      dot++;
      if(dot%frame==0) {
        dot=0;
        if(fr.nextFrame(current)) fr.workPlane(plane,frame);
        current=new char[frame];
      }
    }
    writePlane(plane,name,res);
  }
  /** Reads a char <CODE>plane</CODE> of size <CODE>res*res*res</CODE> from the <CODE>name</CODE> file
   *
   * @return returns false if file can't be opened
   * @param plane 2D char array
   * @param name file to read
   * @param res size of cube sides
   */
  public static boolean readPlane(char[][] plane, String name, int res) {
    try {
      FileInputStream align=new FileInputStream(name);
      for(int x=0;x<res;x++) {
        for(int y=0;y<res;y++) {
          char b=(char)align.read();
          plane[x][y]=b;
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }
  /** writes <CODE>plane</CODE> to <CODE>name</CODE> file
   * @param plane 2D char array
   * @param name file name
   * @param res size of cube sides
   */
  public static void writePlane(char[][] plane, String name, int res) {
    try {
      FileOutputStream align=new FileOutputStream(name);
      for(int x=0;x<res;x++) {
        for(int y=0;y<res;y++) {
          align.write(plane[x][y]);
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}