/*
 * ShowCube.java
 *
 * Created on June 20, 2002, 5:41 PM
 */
import javax.media.j3d.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.applet.*;
import javax.vecmath.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
/** an applet with main method to run from command line. reads "align.aplane"
 * and displays it in 3D. requires Java 3D runtime.
 * @author Dmitri Pavlenkov
 */
public class ShowPlane extends javax.swing.JPanel {
  private static String filename="align.aplane";
  private static boolean main=true;
  /** A constructor for main method
   * @param name file name of 2D fractal index.
   */  
  public static void setFile(String name) {
    filename=name;
  }
  /** Creates a JPanel with Canvas3D
   */  
  public ShowPlane() {
    setLayout(new BorderLayout());
    GraphicsConfiguration config=SimpleUniverse.getPreferredConfiguration();
    Canvas3D canvas3D=new Canvas3D(config);
    add("Center", canvas3D);
    BranchGroup scene=createSceneGraph();
    SimpleUniverse simpleU=new SimpleUniverse(canvas3D);
    scene.compile();
    simpleU.getViewingPlatform().setNominalViewingTransform();
    simpleU.addBranchGraph(scene);
  }
  /** Convenience method for displaying ShowPlane
   */  
  public static void showFrame() {
    final Frame frame=new Frame("Show Plane");
    frame.add(new ShowPlane());
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {
        frame.dispose();
        if(main) System.exit(0);
      }
    });
    frame.pack();
    frame.setSize(680,680);    
    frame.show();
  }    
    
  /** The function that loads plane file and colors the dots accordingly to the
   * number of bits set at every location. first three bits are red, second three
   * are green, and last two reverse bits are blue
   * @return cube vertexes rotating 360degrees/min rotated 22.5 degrees around X axis
   */
  public BranchGroup createSceneGraph() {
    BranchGroup objRoot=new BranchGroup();
    Transform3D rotate=new Transform3D();
    rotate.rotX(Math.PI/8.0d);
    Transform3D temprotate=new Transform3D();
    temprotate.rotY(Math.PI/5.0d);
    //rotate.mul(temprotate);
    TransformGroup objRotate=new TransformGroup(rotate);
    TransformGroup objSpin=new TransformGroup();
    objSpin.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    objRoot.addChild(objRotate);
    objRotate.addChild((objSpin));
    int res=256;
    Vector[] coords=new Vector[res*2];
    try {
      File f=new File(filename);
      if(f.length()==128*128) res=128; else res=256;
      FileInputStream align=new FileInputStream(f);
      for(int x=0;x<res;x++) {
        //if(cube[x][x][x]>0)
        //System.out.println(FullCube.showBinary(new char[] {cube[x][x][x], (char)x})+((char)x)+" x="+x+" cube[x][x][x]="+((int)cube[x][x][x]));
        for(int y=0;y<res;y++) {
          int c=align.read();
          int location=-1;
          if((x%(res-1)==0)||(y%(res-1)==0)) {
            location=0;
          }
          if(c>0) {
            location=c;
          }
          if(location>-1) {
            if(coords[location]==null) {
              coords[location]=new Vector();
            }
            coords[location].add(new Point3f((x-(res-1)/2.0f)/(res-1),(y-(res-1)/2.0f)/(res-1),0));
          }
          
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    
    for(int i=0;i<res*2;i++) {
      if(i==0&&coords[i]!=null) {
        System.out.println("Vector "+i+" size "+coords[i].size());
        GeometryArray point=new PointArray(coords[i].size(),GeometryArray.COORDINATES|GeometryArray.COLOR_3);
        for(int j=0;j<coords[0].size();j++) {
          Point3f p=(Point3f)coords[0].get(j);
          //          Appearance app=new Appearance();
          //          app.setColoringAttributes(
          //          new ColoringAttributes(p.x+0.5f,p.y+0.5f,p.z+0.5f, ColoringAttributes.FASTEST));
          if(p.x*p.x>0.5f*0.5f||p.y*p.y>0.5f*0.5f||p.z*p.z>0.5f*0.5f) System.out.println("Error "+p.toString());
          point.setCoordinate(j,p);
          point.setColor(j, new Color3f(p.x+0.5f,p.y+0.5f,0));
        }
        Shape3D shape=new Shape3D(point);
        //      shape.setAppearance(app);
        objSpin.addChild(shape);
      } else
        
        if(coords[i]!=null) {
          
          Appearance app=new Appearance();
          app.setColoringAttributes(
          new ColoringAttributes(((float)(i%8))/7.0f,((float)(i%64))/63.0f,((float)(i%256))/255.0f,
          ColoringAttributes.FASTEST));
          System.out.println("Vector "+i+" size "+coords[i].size());
/*
          if(coords[i].size()<1) continue;
          if(coords[i].size()<2) {
            GeometryArray point=new PointArray(1,GeometryArray.COORDINATES);
            point.setCoordinate(0,(Point3f)coords[i].get(0));
            Shape3D shape=new Shape3D();
            shape.setAppearance(app);
            objSpin.addChild(shape);
            continue;
          }
 */
          int[] sep=new int[1];
          sep[0]=coords[i].size();
          //GeometryArray lines=new LineStripArray(coords[i].size(), GeometryArray.COORDINATES,sep);
          GeometryArray lines=new PointArray(coords[i].size(), GeometryArray.COORDINATES);
          for(int j=0;j<coords[i].size();j++) {
            lines.setCoordinate(j,(Point3f)coords[i].get(j));
          }
          Shape3D shape=new Shape3D(lines);
          shape.setAppearance(app);
          objSpin.addChild(shape);
        }
    }
    Transform3D yAxis=new Transform3D();
    Alpha rotationAlpha=new Alpha(-1,60000);
    RotationInterpolator rotator=
    new RotationInterpolator(rotationAlpha,objSpin,yAxis,0.0f,(float)Math.PI*2.0f);
    BoundingSphere bounds=new BoundingSphere();
    rotator.setSchedulingBounds(bounds);
    objSpin.addChild(rotator);
    return objRoot;
  }
  /** takes an optional alignfile argument, otherwise defaults to "align.aplane"
   * @param args alignfile
   */
  public static void main(String[] args) {
    if(args.length>0) {
      filename=args[0];
    }
    //Frame frame=new MainFrame(new ShowPlane(), 680, 680);
    showFrame();
  }
  /** Initializes the applet ShowPlane
   */
  public void init() {
    initComponents();
  }
  
  /** This method is called from within the init() method to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  private void initComponents() {//GEN-BEGIN:initComponents
    
    setLayout(new java.awt.BorderLayout());
    
  }//GEN-END:initComponents
  
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  // End of variables declaration//GEN-END:variables
  
}
