package net.fractalkiss.util;

/*
 * Util.java
 *
 * Created on June 27, 2002, 8:32 PM
 */

/**
 *
 * @author  Dmitri Pavlenkov
 * @version
 */
import java.io.*;

/** Base Utility class
 */
public class Util extends Object {
  
  
  /** performs NAND operation (NotAnd), it's the commutative opposite of andNot,
   * but sounds and writes shorter. The thing of notice, however is that NAND
   * is not commutative, xor mask is negated and then compared to presentation
   * @param sb presentation
   * @param xor xor mask
   * @return nand result, essentially fragmented sb
   */
  public static char[] nand(char[] sb, char[] xor) {
    char[] nand=new char[sb.length];
    for(int i=0;i<sb.length;i++) {
      nand[i]=(char)((~(int)xor[i])&((int)sb[i]));
    }
    return nand;
  }
  public static char[] and(char[] sb, char[] xor) {
    char[] and=new char[sb.length];
    for(int i=0;i<sb.length;i++) {
      and[i]=(char)(xor[i]&sb[i]);
    }
    return and;
  }
    public static char[] or(char[] sb, char[] xor) {
    char[] or=new char[sb.length];
    for(int i=0;i<sb.length;i++) {
      or[i]=(char)(xor[i]|sb[i]);
    }
    return or;
  }

  /** per byte XOR of two char arrays
   * @param a array 1
   * @param b array 2
   * @return xor result
   */
  public static char[] xor(char[] a, char[] b) {
    int len=a.length;
    char[] xor=new char[len];
    for(int i=0;i<len;i++) {
      xor[i]=(char)(((int)a[i])^((int)b[i]));
    }
    //System.out.println(new String(a)+"^"+new String(b)+"="+new String(xor));
    return xor;
  }
  /** fills 2D char array with zeros
   * @param plane char array to fill
   * @param x size of first index
   * @param y size of second index
   */
  public static void fill02(char[][] plane, int x, int y) {
    for(int i=0;i<x;i++) {
      for(int j=0;j<y;j++) {
        plane[i][j]=(char)0;
      }
    }
  }
  /** fills 3D char array with zeros
   * @param cube 3D char array
   * @param x size of first index
   * @param y size of second index
   * @param z size of third index
   */
  public static void fill03(char[][][] cube, int x, int y, int z) {
    for(int i=0;i<x;i++) {
      for(int j=0;j<y;j++) {
        for(int k=0;k<z;k++) {
          cube[i][j][k]=(char)0;
        }
      }
    }
  }
  /** Makes sure chars are written without encoding
   * @param ou outputstream
   * @param level fragmentation level
   */
  public static void writeBytes(OutputStream ou, char[] level) {
    try {
      for(int i=0;i<level.length;i++) {
        ou.write((int)level[i]);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  /** writes <CODE>plane</CODE> to <CODE>name</CODE> file
   * @param plane 2D char array
   * @param name file name
   * @param res size of cube sides
   */
  public static void writePlane(char[][] plane, String name, int res) {
    try {
      FileOutputStream align=new FileOutputStream(name);
      for(int x=0;x<res;x++) {
        for(int y=0;y<res;y++) {
          align.write(plane[x][y]);
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }  
}
